import asyncio
import datetime
from typing import (
    Any,
    ClassVar,
    Coroutine,
    Dict,
    Generic,
    Iterable,
    Optional,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Literal

import aiohttp

from .abc import User as _ABCUser
from .asset import Asset
from .channel import TextChannel
from .embeds import Embed
from .enums import WebhookType
from .file import File
from .guild import Guild
from .http import _WebhookDict
from .mentions import AllowedMentions
from .message import Message
from .mixins import Hashable
from .state import ConnectionState

_T = TypeVar('_T')

_AsyncNone = Coroutine[Any, Any, None]
_AsyncOptionalMessage = Coroutine[Any, Any, Optional[WebhookMessage[_AsyncNone]]]

_N = TypeVar('_N', bound=Union[_AsyncNone, None])
_M = TypeVar('_M', bound=Union[_AsyncOptionalMessage, Optional[WebhookMessage[None]]])

class WebhookAdapter(Generic[_N, _M]):
    BASE: ClassVar[str]

    webhook: Webhook[_N, _M]
    def is_async(self) -> bool: ...
    def request(
        self,
        verb: str,
        url: str,
        payload: Optional[Dict[str, Any]] = ...,
        multipart: Optional[Dict[str, Any]] = ...,
    ) -> Any: ...
    def delete_webhook(self, *, reason: Optional[str] = ...) -> _N: ...
    def edit_webhook(
        self,
        *,
        reason: Optional[str] = ...,
        **payload: Any,
    ) -> _N: ...
    def edit_webhook_message(self, message_id: int, payload: Any) -> _N: ...
    def delete_webhook_message(self, message_id: int) -> _N: ...
    def handle_execution_response(
        self,
        response: Any,
        *,
        wait: bool,
    ) -> _M: ...
    def execute_webhook(
        self,
        *,
        payload: Dict[str, Any],
        wait: bool = ...,
        file: Optional[File] = ...,
        files: Optional[Iterable[File]] = ...,
    ) -> _M: ...

class AsyncWebhookAdapter(WebhookAdapter[_AsyncNone, _AsyncOptionalMessage]):
    session: aiohttp.ClientSession
    loop: asyncio.AbstractEventLoop
    def __init__(self, session: aiohttp.ClientSession) -> None: ...
    def is_async(self) -> bool: ...
    async def request(
        self,
        verb: str,
        url: str,
        payload: Optional[Dict[str, Any]] = ...,
        multipart: Optional[Dict[str, Any]] = ...,
        *,
        files: Optional[Iterable[File]] = ...,
        reason: Optional[str] = ...,
    ) -> Optional[WebhookMessage[_AsyncNone]]: ...

class RequestsWebhookAdapter(WebhookAdapter[None, WebhookMessage[None]]):
    session: Any
    def __init__(self, session: Optional[Any] = ..., *, sleep: bool = ...) -> None: ...
    def request(
        self,
        verb: str,
        url: str,
        payload: Optional[Dict[str, Any]] = ...,
        multipart: Optional[Dict[str, Any]] = ...,
        *,
        files: Optional[Iterable[File]] = ...,
        reason: Optional[str] = ...,
    ) -> WebhookMessage[None]: ...

class WebhookMessage(Message, Generic[_N]):
    @overload  # type: ignore[override]
    def edit(
        self,
        *,
        content: Optional[str] = ...,
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _N: ...
    @overload
    def edit(
        self,
        *,
        content: Optional[str] = ...,
        embeds: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _N: ...

class Webhook(Hashable, Generic[_N, _M]):
    id: int
    type: WebhookType
    token: Optional[str]
    channel_id: Optional[int]
    guild_id: Optional[int]
    name: Optional[str]
    avatar: Optional[str]
    user: Optional[_ABCUser]
    @property
    def url(self) -> str: ...
    @classmethod
    def partial(
        cls, id: int, token: str, *, adapter: WebhookAdapter[_N, _M]
    ) -> Webhook[_N, _M]: ...
    @classmethod
    def from_url(
        cls, url: str, *, adapter: WebhookAdapter[_N, _M]
    ) -> Webhook[_N, _M]: ...
    # NOTE: While this method is public, it should never be invoked by users.
    @classmethod
    def from_state(
        cls, data: _WebhookDict, state: ConnectionState
    ) -> _AsyncWebhook: ...
    @property
    def guild(self) -> Optional[Guild]: ...
    @property
    def channel(self) -> Optional[TextChannel]: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def avatar_url(self) -> Asset: ...
    def avatar_url_as(
        self, *, format: Optional[Literal['png', 'jpg', 'jpeg']] = ..., size: int = ...
    ) -> Asset: ...
    def delete(self, *, reason: Optional[str] = ...) -> _N: ...
    def edit(
        self,
        *,
        reason: Optional[str] = ...,
        name: Optional[str] = ...,
        avatar: Optional[bytes] = ...,
    ) -> _N: ...
    @overload
    def send(
        self,
        content: str,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: Optional[File] = ...,
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: str,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: Optional[File] = ...,
        embeds: Optional[Iterable[Embed]] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: str,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Optional[Iterable[File]] = ...,
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: str,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Optional[Iterable[File]] = ...,
        embeds: Optional[Iterable[Embed]] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: File,
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: File,
        embeds: Optional[Iterable[Embed]] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Iterable[File],
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Iterable[File],
        embeds: Optional[Iterable[Embed]] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: Optional[File] = ...,
        embed: Embed,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Optional[Iterable[File]] = ...,
        embed: Embed,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: Optional[File] = ...,
        embeds: Iterable[Embed],
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Optional[Iterable[File]] = ...,
        embeds: Iterable[Embed],
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _M: ...
    execute = send
    @overload
    def edit_message(
        self,
        message_id: int,
        *,
        content: Optional[str] = ...,
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _N: ...
    @overload
    def edit_message(
        self,
        message_id: int,
        *,
        content: Optional[str] = ...,
        embeds: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> _N: ...
    def delete_message(self, message_id: int) -> _N: ...

_AsyncWebhook = Webhook[_AsyncNone, _AsyncOptionalMessage]



from typing import List, Optional, Set, type_check_only
from typing_extensions import Literal, TypedDict

from .http import _MessageDict, _PartialEmojiDict
from .member import Member
from .message import Message
from .partial_emoji import PartialEmoji

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _BaseBulkMessageDeleteDict(TypedDict, total=False):
    guild_id: str

@type_check_only
class _BulkMessageDeleteDict(_BaseBulkMessageDeleteDict):
    ids: List[str]
    channel_id: str

@type_check_only
class _BaseReactionActionDict(TypedDict):
    message_id: str
    channel_id: str
    user_id: str
    emoji: _PartialEmojiDict

@type_check_only
class _ReactionActionDict(_BaseReactionActionDict, total=False):
    guild_id: str

@type_check_only
class _BaseReactionClearDict(TypedDict):
    channel_id: int
    message_id: int

@type_check_only
class _ReactionClearDict(_BaseReactionClearDict, total=False):
    guild_id: int

class _RawReprMixin: ...

class RawMessageDeleteEvent(_RawReprMixin):
    message_id: int
    channel_id: int
    guild_id: Optional[int]
    cached_message: Optional[Message]
    def __init__(self, data: _MessageDict) -> None: ...

class RawBulkMessageDeleteEvent(_RawReprMixin):
    message_ids: Set[int]
    channel_id: int
    guild_id: Optional[int]
    cached_messages: List[Message]
    def __init__(self, data: _BulkMessageDeleteDict) -> None: ...

class RawMessageUpdateEvent(_RawReprMixin):
    message_id: int
    channel_id: int
    guild_id: Optional[int]
    data: _MessageDict
    cached_message: Optional[Message]
    def __init__(self, data: _MessageDict) -> None: ...

class RawReactionActionEvent(_RawReprMixin):
    message_id: int
    channel_id: int
    user_id: int
    emoji: PartialEmoji
    member: Optional[Member]
    event_type: Literal['REACTION_ADD', 'REACTION_REMOVE']
    guild_id: Optional[int]
    def __init__(
        self,
        data: _ReactionActionDict,
        emoji: PartialEmoji,
        event_type: Literal['REACTION_ADD', 'REACTION_REMOVE'],
    ) -> None: ...

class RawReactionClearEvent(_RawReprMixin):
    message_id: int
    channel_id: int
    guild_id: Optional[int]
    def __init__(self, data: _ReactionClearDict) -> None: ...

class RawReactionClearEmojiEvent(_RawReprMixin):
    message_id: int
    channel_id: int
    guild_id: Optional[int]
    emoji: PartialEmoji
    def __init__(self, data: _ReactionClearDict, emoji: PartialEmoji) -> None: ...



from typing import Any, Iterable, List, Type, TypeVar, Union, type_check_only
from typing_extensions import TypedDict

from .abc import Snowflake

_AM = TypeVar('_AM', bound=AllowedMentions)

class _FakeBool:
    def __eq__(self, other: Any) -> bool: ...
    def __bool__(self) -> bool: ...

default: _FakeBool

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _AllowedMentionsDictBase(TypedDict):
    parse: List[str]

@type_check_only
class _AllowedMentionsDict(_AllowedMentionsDictBase, total=False):
    users: List[int]
    roles: List[int]
    replied_user: bool

class AllowedMentions:
    everyone: bool
    users: Union[bool, Iterable[Snowflake]]
    roles: Union[bool, Iterable[Snowflake]]
    replied_user: bool
    def __init__(
        self,
        *,
        everyone: Union[bool, _FakeBool] = ...,
        users: Union[bool, _FakeBool, Iterable[Snowflake]] = ...,
        roles: Union[bool, _FakeBool, Iterable[Snowflake]] = ...,
        replied_user: Union[bool, _FakeBool] = ...,
    ) -> None: ...
    @classmethod
    def all(cls: Type[_AM]) -> _AM: ...
    @classmethod
    def none(cls: Type[_AM]) -> _AM: ...
    def to_dict(self) -> _AllowedMentionsDict: ...
    def merge(self, other: AllowedMentions) -> AllowedMentions: ...



from typing import Any, Callable, Coroutine, Generic, List, Optional, TypeVar, Union
from typing_extensions import Final

from .audit_logs import AuditLogEntry
from .guild import Guild
from .http import _GuildDict, _GuildMemberDict
from .member import Member
from .message import Message
from .object import Object
from .user import User

OLDEST_OBJECT: Final[Object]

_IT = TypeVar('_IT')
_AIT = TypeVar('_AIT', bound=_AsyncIterator[Any])
_OIT = TypeVar('_OIT')
_VT = TypeVar('_VT')

class _AsyncIterator(Generic[_IT]):
    def get(self, **attrs: Any) -> Coroutine[Any, Any, Optional[_IT]]: ...
    async def find(
        self, predicate: Callable[[_IT], Union[bool, Coroutine[Any, Any, bool]]]
    ) -> Optional[_IT]: ...
    def chunk(self, max_size: int) -> _ChunkedAsyncIterator[_IT]: ...
    def map(
        self, func: Callable[[_IT], Union[_OIT, Coroutine[Any, Any, _OIT]]]
    ) -> _MappedAsyncIterator[_OIT]: ...
    def filter(
        self, predicate: Callable[[_IT], Union[bool, Coroutine[Any, Any, bool]]]
    ) -> _FilteredAsyncIterator[_IT]: ...
    async def flatten(self) -> List[_IT]: ...
    def __aiter__(self: _AIT) -> _AIT: ...
    async def __anext__(self) -> _IT: ...

def _identity(x: _VT) -> _VT: ...

class _ChunkedAsyncIterator(_AsyncIterator[List[_IT]]):
    async def next(self) -> List[_IT]: ...

class _MappedAsyncIterator(_AsyncIterator[_IT]):
    async def next(self) -> _IT: ...

class _FilteredAsyncIterator(_AsyncIterator[_IT]):
    async def next(self) -> _IT: ...

class ReactionIterator(_AsyncIterator[Union[User, Member]]):
    async def next(self) -> Union[User, Member]: ...
    async def fill_users(self) -> None: ...

class HistoryIterator(_AsyncIterator[Message]):
    async def next(self) -> Message: ...
    async def flatten(self) -> List[Message]: ...
    async def fill_messages(self) -> None: ...

class AuditLogIterator(_AsyncIterator[AuditLogEntry]):
    async def next(self) -> AuditLogEntry: ...

class GuildIterator(_AsyncIterator[Guild]):
    async def next(self) -> Guild: ...
    def create_guild(self, data: _GuildDict) -> Guild: ...
    async def flatten(self) -> List[Guild]: ...
    async def fill_guilds(self) -> None: ...

class MemberIterator(_AsyncIterator[Member]):
    async def next(self) -> Member: ...
    async def fill_members(self) -> None: ...
    async def create_member(self, data: _GuildMemberDict) -> Member: ...



from inspect import Parameter
from typing import (
    Any,
    Callable,
    Coroutine,
    Dict,
    Generic,
    Iterator,
    List,
    Mapping,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    ValuesView,
    overload,
    type_check_only,
)
from typing_extensions import Protocol

from ._types import _BaseCommand
from .cog import Cog
from .context import Context
from .cooldowns import Cooldown, CooldownMapping, _CooldownType

_CoroType = Callable[..., Coroutine[Any, Any, Any]]

_CT = TypeVar('_CT', bound=Context)
_C = TypeVar('_C', bound=_CoroType)
_CMD = TypeVar('_CMD', bound=Command[Any])
_F = TypeVar('_F', bound=Union[_CoroType, Command[Any]])
_T_contra = TypeVar('_T_contra', contravariant=True)

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _CheckPredicate(Protocol[_T_contra]):
    def __call__(self, __ctx: _T_contra) -> Union[bool, Coroutine[Any, Any, bool]]: ...

@type_check_only
class _CheckDecorator(Protocol):
    def __call__(self, __func: _F) -> _F: ...

@type_check_only
class _InvokeCallback(Protocol[_T_contra]):
    async def __call__(self, __ctx: _T_contra) -> None: ...

class Command(_BaseCommand, Generic[_CT]):
    name: str
    callback: _CoroType
    help: Optional[str]
    brief: Optional[str]
    usage: Optional[str]
    aliases: Union[List[str], Tuple[str, ...]]
    enabled: bool
    parent: Optional[Command[_CT]]
    checks: List[_CheckPredicate[_CT]]
    description: str
    hidden: bool
    rest_is_raw: bool
    ignore_extra: bool
    require_var_positional: bool
    cooldown_after_parsing: bool
    params: Mapping[str, Parameter]
    _buckets: CooldownMapping
    cog: Optional[Cog[_CT]]
    def __init__(
        self,
        func: _CoroType,
        *,
        name: str = ...,
        enabled: bool = ...,
        help: Optional[str] = ...,
        brief: Optional[str] = ...,
        usage: Optional[str] = ...,
        aliases: Union[List[str], Tuple[str, ...]] = ...,
        description: str = ...,
        hidden: bool = ...,
        rest_is_raw: bool = ...,
        ignore_extra: bool = ...,
        require_var_positional: bool = ...,
        cooldown_after_parsing: bool = ...,
        checks: List[_CheckPredicate[_CT]] = ...,
        cooldown: Cooldown = ...,
        parent: _BaseCommand = ...,
        cog: Optional[Cog[_CT]] = ...,
    ) -> None: ...
    def add_check(self, func: _CheckPredicate[_CT]) -> None: ...
    def remove_check(self, func: _CheckPredicate[_CT]) -> None: ...
    def update(
        self,
        *,
        name: str = ...,
        enabled: bool = ...,
        help: Optional[str] = ...,
        brief: Optional[str] = ...,
        usage: Optional[str] = ...,
        aliases: Union[List[str], Tuple[str, ...]] = ...,
        description: str = ...,
        hidden: bool = ...,
        rest_is_raw: bool = ...,
        ignore_extra: bool = ...,
        require_var_positional: bool = ...,
        cooldown_after_parsing: bool = ...,
    ) -> None: ...
    async def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def copy(self: _CMD) -> _CMD: ...
    async def dispatch_error(self, ctx: _CT, error: Exception) -> None: ...
    async def do_conversion(
        self, ctx: _CT, converter: Any, argument: str, param: Parameter
    ) -> Any: ...
    async def transform(self, ctx: _CT, param: Parameter) -> Any: ...
    @property
    def clean_params(self) -> Mapping[str, Parameter]: ...
    @property
    def full_parent_name(self) -> str: ...
    @property
    def parents(self) -> List[Command[_CT]]: ...
    @property
    def root_parent(self) -> Optional[Command[_CT]]: ...
    @property
    def qualified_name(self) -> str: ...
    async def call_before_hooks(self, ctx: _CT) -> None: ...
    async def call_after_hooks(self, ctx: _CT) -> None: ...
    async def prepare(self, ctx: _CT) -> None: ...
    def is_on_cooldown(self, ctx: _CT) -> bool: ...
    def reset_cooldown(self, ctx: _CT) -> None: ...
    def get_cooldown_retry_after(self, ctx: _CT) -> float: ...
    async def invoke(self, ctx: _CT) -> None: ...
    async def reinvoke(self, ctx: _CT, *, call_hooks: bool = ...) -> None: ...
    def error(self, coro: _C) -> _C: ...
    def has_error_handler(self) -> bool: ...
    def before_invoke(self, coro: _C) -> _C: ...
    def after_invoke(self, coro: _C) -> _C: ...
    @property
    def cog_name(self) -> Optional[str]: ...
    @property
    def short_doc(self) -> str: ...
    @property
    def signature(self) -> str: ...
    async def can_run(self, ctx: _CT) -> bool: ...

class GroupMixin(Generic[_CT]):
    all_commands: Dict[str, Command[_CT]]
    case_insensitive: bool
    @property
    def commands(self) -> ValuesView[Command[_CT]]: ...
    def recursively_remove_all_commands(self) -> None: ...
    def add_command(self, command: Command[_CT]) -> None: ...
    def remove_command(self, name: str) -> Optional[Command[_CT]]: ...
    def walk_commands(self) -> Iterator[Union[Command[_CT], Group[_CT]]]: ...
    def get_command(self, name: str) -> Optional[Command[_CT]]: ...
    def command(
        self,
        name: Optional[str] = ...,
        *,
        enabled: bool = ...,
        help: Optional[str] = ...,
        brief: Optional[str] = ...,
        usage: Optional[str] = ...,
        aliases: Union[List[str], Tuple[str, ...]] = ...,
        description: str = ...,
        hidden: bool = ...,
        rest_is_raw: bool = ...,
        ignore_extra: bool = ...,
        require_var_positional: bool = ...,
        cooldown_after_parsing: bool = ...,
    ) -> Callable[[_CoroType], Command[_CT]]: ...
    def group(
        self,
        name: str = ...,
        *,
        enabled: bool = ...,
        help: Optional[str] = ...,
        brief: Optional[str] = ...,
        usage: Optional[str] = ...,
        aliases: Union[List[str], Tuple[str, ...]] = ...,
        description: str = ...,
        hidden: bool = ...,
        rest_is_raw: bool = ...,
        ignore_extra: bool = ...,
        cooldown_after_parsing: bool = ...,
        invoke_without_command: bool = ...,
        case_insensitive: bool = ...,
    ) -> Callable[[_CoroType], Group[_CT]]: ...

_G = TypeVar('_G', bound=Group[Any])

class Group(GroupMixin[_CT], Command[_CT]):
    invoke_without_command: bool
    def __init__(
        self,
        *,
        name: str = ...,
        enabled: bool = ...,
        help: Optional[str] = ...,
        brief: Optional[str] = ...,
        usage: Optional[str] = ...,
        aliases: Union[List[str], Tuple[str, ...]] = ...,
        description: str = ...,
        hidden: bool = ...,
        rest_is_raw: bool = ...,
        ignore_extra: bool = ...,
        cooldown_after_parsing: bool = ...,
        invoke_without_command: bool = ...,
        case_insensitive: bool = ...,
    ) -> None: ...
    def copy(self: _G) -> _G: ...

@overload
def command(
    name: Optional[str] = ...,
    *,
    enabled: bool = ...,
    help: Optional[str] = ...,
    brief: Optional[str] = ...,
    usage: Optional[str] = ...,
    aliases: Union[List[str], Tuple[str, ...]] = ...,
    description: str = ...,
    hidden: bool = ...,
    rest_is_raw: bool = ...,
    ignore_extra: bool = ...,
    require_var_positional: bool = ...,
    cooldown_after_parsing: bool = ...,
) -> Callable[[_CoroType], Command[Any]]: ...
@overload
def command(
    name: Optional[str] = ...,
    cls: Optional[Type[Command[_CT]]] = ...,
    *,
    enabled: bool = ...,
    help: Optional[str] = ...,
    brief: Optional[str] = ...,
    usage: Optional[str] = ...,
    aliases: Union[List[str], Tuple[str, ...]] = ...,
    description: str = ...,
    hidden: bool = ...,
    rest_is_raw: bool = ...,
    ignore_extra: bool = ...,
    require_var_positional: bool = ...,
    cooldown_after_parsing: bool = ...,
) -> Callable[[_CoroType], Command[_CT]]: ...
@overload
def group(
    name: str = ...,
    *,
    enabled: bool = ...,
    help: Optional[str] = ...,
    brief: Optional[str] = ...,
    usage: Optional[str] = ...,
    aliases: Union[List[str], Tuple[str, ...]] = ...,
    description: str = ...,
    hidden: bool = ...,
    rest_is_raw: bool = ...,
    ignore_extra: bool = ...,
    cooldown_after_parsing: bool = ...,
    invoke_without_command: bool = ...,
    case_insensitive: bool = ...,
) -> Callable[[_CoroType], Group[Any]]: ...
@overload
def group(
    name: str = ...,
    *,
    cls: Optional[Type[Group[_CT]]] = ...,
    enabled: bool = ...,
    help: Optional[str] = ...,
    brief: Optional[str] = ...,
    usage: Optional[str] = ...,
    aliases: Union[List[str], Tuple[str, ...]] = ...,
    description: str = ...,
    hidden: bool = ...,
    rest_is_raw: bool = ...,
    ignore_extra: bool = ...,
    cooldown_after_parsing: bool = ...,
    invoke_without_command: bool = ...,
    case_insensitive: bool = ...,
) -> Callable[[_CoroType], Group[Any]]: ...
def check(predicate: _CheckPredicate[_CT]) -> _CheckDecorator: ...
def check_any(*checks: _CheckPredicate[_CT]) -> _CheckDecorator: ...
def has_role(item: Union[int, str]) -> _CheckDecorator: ...
def has_any_role(*items: Union[int, str]) -> _CheckDecorator: ...
def bot_has_role(item: Union[int, str]) -> _CheckDecorator: ...
def bot_has_any_role(*items: Union[int, str]) -> _CheckDecorator: ...
def has_permissions(**perms: bool) -> _CheckDecorator: ...
def bot_has_permissions(**perms: bool) -> _CheckDecorator: ...
def has_guild_permissions(**perms: bool) -> _CheckDecorator: ...
def bot_has_guild_permissions(**perms: bool) -> _CheckDecorator: ...
def dm_only() -> _CheckDecorator: ...
def guild_only() -> _CheckDecorator: ...
def is_owner() -> _CheckDecorator: ...
def is_nsfw() -> _CheckDecorator: ...
def cooldown(rate: int, per: float, type: _CooldownType = ...) -> _CheckDecorator: ...
def max_concurrency(
    number: int, per: _CooldownType = ..., *, wait: bool = ...
) -> _CheckDecorator: ...
def before_invoke(coro: _InvokeCallback[_CT]) -> _CheckDecorator: ...
def after_invoke(coro: _InvokeCallback[_CT]) -> _CheckDecorator: ...



import datetime
from typing import Any, Optional

from .enums import VoiceRegion
from .guild import Guild
from .user import User

class Template:
    code: str
    uses: int
    name: str
    description: Optional[str]
    creator: User
    created_at: Optional[datetime.datetime]
    updated_at: Optional[datetime.datetime]
    source_guild: Guild
    async def create_guild(
        self, name: str, region: Optional[VoiceRegion] = ..., icon: Optional[Any] = ...
    ) -> Guild: ...
    async def sync(self) -> None: ...
    async def edit(
        self, *, name: str = ..., description: Optional[str] = ...
    ) -> None: ...
    async def delete(self) -> None: ...



import asyncio
from datetime import datetime
from typing import (
    Any,
    Callable,
    Coroutine,
    Iterator,
    List,
    Optional,
    Tuple,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Literal

import aiohttp

from .abc import Snowflake
from .activity import BaseActivity
from .appinfo import AppInfo
from .channel import (
    CategoryChannel,
    DMChannel,
    GroupChannel,
    StoreChannel,
    TextChannel,
    VoiceChannel,
)
from .emoji import Emoji
from .enums import Status, VoiceRegion
from .flags import Intents, MemberCacheFlags
from .gateway import DiscordWebSocket
from .guild import Guild
from .invite import Invite
from .iterators import GuildIterator
from .member import Member, VoiceState
from .mentions import AllowedMentions
from .message import Message
from .raw_models import (
    RawBulkMessageDeleteEvent,
    RawMessageDeleteEvent,
    RawMessageUpdateEvent,
    RawReactionActionEvent,
    RawReactionClearEmojiEvent,
    RawReactionClearEvent,
)
from .reaction import Reaction
from .relationship import Relationship
from .role import Role
from .template import Template
from .user import ClientUser, Profile, User
from .utils import SequenceProxy
from .voice_client import VoiceProtocol
from .webhook import _AsyncWebhook
from .widget import Widget

_FuncType = Callable[..., Coroutine[Any, Any, Any]]
_F = TypeVar('_F', bound=_FuncType)
_GuildChannel = Union[TextChannel, VoiceChannel, CategoryChannel, StoreChannel]
_PrivateChannel = Union[DMChannel, GroupChannel]
_User = Union[User, Member]

class Client:
    ws: Optional[DiscordWebSocket]
    loop: asyncio.AbstractEventLoop
    shard_id: Optional[int]
    shard_count: Optional[int]
    activity: Optional[BaseActivity]
    allowed_mentions: Optional[AllowedMentions]
    def __init__(
        self,
        *,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        shard_id: Optional[int] = ...,
        shard_count: Optional[int] = ...,
        intents: Optional[Intents] = ...,
        member_cache_flags: Optional[MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[Status] = ...,
        activity: Optional[BaseActivity] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @property
    def latency(self) -> float: ...
    def is_ws_ratelimited(self) -> bool: ...
    # NOTE: user is actually Optional[ClientUser] because it's None when logged out, but the vast
    # majority of uses will be while logged in. Because of this fact, it is typed as ClientUser to
    # prevent false positives
    @property
    def user(self) -> ClientUser: ...
    @property
    def guilds(self) -> List[Guild]: ...
    @property
    def emojis(self) -> List[Emoji]: ...
    @property
    def cached_messages(self) -> SequenceProxy[Message]: ...
    @property
    def private_channels(self) -> List[_PrivateChannel]: ...
    @property
    def voice_clients(self) -> List[VoiceProtocol]: ...
    def is_ready(self) -> bool: ...
    def dispatch(self, __event: str, *args: Any, **kwargs: Any) -> None: ...
    async def on_error(self, event_method: str, *args: Any, **kwargs: Any) -> None: ...
    async def request_offline_members(self, *guilds: Guild) -> None: ...
    async def before_identify_hook(
        self, shard_id: int, *, initial: bool = ...
    ) -> None: ...
    async def login(self, token: str, *, bot: bool = ...) -> None: ...
    async def logout(self) -> None: ...
    async def connect(self, *, reconnect: bool = ...) -> None: ...
    async def close(self) -> None: ...
    def clear(self) -> None: ...
    async def start(
        self, token: str, *, bot: bool = ..., reconnect: bool = ...
    ) -> None: ...
    def run(self, token: str, *, bot: bool = ..., reconnect: bool = ...) -> None: ...
    def is_closed(self) -> bool: ...
    @property
    def intents(self) -> Intents: ...
    @property
    def users(self) -> List[User]: ...
    def get_channel(
        self, id: int
    ) -> Optional[Union[_GuildChannel, _PrivateChannel]]: ...
    def get_guild(self, id: int) -> Optional[Guild]: ...
    def get_user(self, id: int) -> Optional[User]: ...
    def get_emoji(self, id: int) -> Optional[Emoji]: ...
    def get_all_channels(self) -> Iterator[Union[_GuildChannel]]: ...
    def get_all_members(self) -> Iterator[Member]: ...
    async def wait_until_ready(self) -> None: ...
    @overload
    def wait_for(
        self,
        event: Literal['message'],
        *,
        check: Optional[Callable[[Message], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Message]: ...
    @overload
    def wait_for(
        self,
        event: Literal['message_delete'],
        *,
        check: Optional[Callable[[Message], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Message]: ...
    @overload
    def wait_for(
        self,
        event: Literal['bulk_message_delete'],
        *,
        check: Optional[Callable[[List[Message]], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[List[Message]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['raw_message_delete'],
        *,
        check: Optional[Callable[[RawMessageDeleteEvent], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[RawMessageDeleteEvent]: ...
    @overload
    def wait_for(
        self,
        event: Literal['raw_bulk_message_delete'],
        *,
        check: Optional[Callable[[RawBulkMessageDeleteEvent], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[RawBulkMessageDeleteEvent]: ...
    @overload
    def wait_for(
        self,
        event: Literal['message_edit'],
        *,
        check: Optional[Callable[[Message, Message], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Message, Message]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['raw_message_edit'],
        *,
        check: Optional[Callable[[RawMessageUpdateEvent], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[RawMessageUpdateEvent]: ...
    @overload
    def wait_for(
        self,
        event: Literal['reaction_add'],
        *,
        check: Optional[Callable[[Reaction, Optional[_User]], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Reaction, Optional[_User]]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['raw_reaction_add'],
        *,
        check: Optional[Callable[[RawReactionActionEvent], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[RawReactionActionEvent]: ...
    @overload
    def wait_for(
        self,
        event: Literal['reaction_remove'],
        *,
        check: Optional[Callable[[Reaction, Optional[_User]], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Reaction, Optional[_User]]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['raw_reaction_remove'],
        *,
        check: Optional[Callable[[RawReactionActionEvent], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[RawReactionActionEvent]: ...
    @overload
    def wait_for(
        self,
        event: Literal['reaction_clear'],
        *,
        check: Optional[Callable[[Message, List[Reaction]], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Message, List[Reaction]]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['raw_reaction_clear'],
        *,
        check: Optional[Callable[[RawReactionClearEvent], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[RawReactionClearEvent]: ...
    @overload
    def wait_for(
        self,
        event: Literal['reaction_clear_emoji'],
        *,
        check: Optional[Callable[[Reaction], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Reaction]: ...
    @overload
    def wait_for(
        self,
        event: Literal['raw_reaction_clear_emoji'],
        *,
        check: Optional[Callable[[RawReactionClearEmojiEvent], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[RawReactionClearEmojiEvent]: ...
    @overload
    def wait_for(
        self,
        event: Literal['private_channel_delete'],
        *,
        check: Optional[Callable[[_PrivateChannel], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[_PrivateChannel]: ...
    @overload
    def wait_for(
        self,
        event: Literal['private_channel_create'],
        *,
        check: Optional[Callable[[_PrivateChannel], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[_PrivateChannel]: ...
    @overload
    def wait_for(
        self,
        event: Literal['private_channel_update'],
        *,
        check: Optional[Callable[[_PrivateChannel, _PrivateChannel], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[_PrivateChannel, _PrivateChannel]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['private_channel_pins_update'],
        *,
        check: Optional[Callable[[_PrivateChannel, Optional[datetime]], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[_PrivateChannel, Optional[datetime]]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_channel_delete'],
        *,
        check: Optional[Callable[[_GuildChannel], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[_GuildChannel]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_channel_create'],
        *,
        check: Optional[Callable[[_GuildChannel], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[_GuildChannel]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_channel_update'],
        *,
        check: Optional[Callable[[_GuildChannel, _GuildChannel], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[_GuildChannel, _GuildChannel]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_channel_pins_update'],
        *,
        check: Optional[Callable[[_GuildChannel, Optional[datetime]], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[_GuildChannel, Optional[datetime]]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_integrations_update'],
        *,
        check: Optional[Callable[[Guild], bool]],
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Guild]: ...
    @overload
    def wait_for(
        self,
        event: Literal['webhooks_update'],
        *,
        check: Optional[Callable[[_GuildChannel], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[_GuildChannel]: ...
    @overload
    def wait_for(
        self,
        event: Literal['member_join'],
        *,
        check: Optional[Callable[[Member], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Member]: ...
    @overload
    def wait_for(
        self,
        event: Literal['member_remove'],
        *,
        check: Optional[Callable[[Member], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Member]: ...
    @overload
    def wait_for(
        self,
        event: Literal['member_update'],
        *,
        check: Optional[Callable[[Member, Member], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Member, Member]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['user_update'],
        *,
        check: Optional[Callable[[User, User], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[User, User]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_join'],
        *,
        check: Optional[Callable[[Guild], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Guild]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_remove'],
        *,
        check: Optional[Callable[[Guild], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Guild]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_update'],
        *,
        check: Optional[Callable[[Guild, Guild], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Guild, Guild]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_role_create'],
        *,
        check: Optional[Callable[[Role], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Role]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_role_delete'],
        *,
        check: Optional[Callable[[Role], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Role]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_role_update'],
        *,
        check: Optional[Callable[[Role, Role], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Role, Role]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_emojis_update'],
        *,
        check: Optional[
            Callable[[Guild, Tuple[Emoji, ...], Tuple[Emoji, ...]], bool]
        ] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Guild, Tuple[Emoji, ...], Tuple[Emoji, ...]]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_available'],
        *,
        check: Optional[Callable[[Guild], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Guild]: ...
    @overload
    def wait_for(
        self,
        event: Literal['guild_unavailable'],
        *,
        check: Optional[Callable[[Guild], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Guild]: ...
    @overload
    def wait_for(
        self,
        event: Literal['voice_state_update'],
        *,
        check: Optional[Callable[[Member, VoiceState, VoiceState], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Member, VoiceState, VoiceState]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['member_ban'],
        *,
        check: Optional[Callable[[Guild, _User], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Guild, _User]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['member_unban'],
        *,
        check: Optional[Callable[[Guild, User], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Guild, User]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['group_join'],
        *,
        check: Optional[Callable[[GroupChannel, User], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[GroupChannel, User]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['group_remove'],
        *,
        check: Optional[Callable[[GroupChannel, User], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[GroupChannel, User]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['relationship_add'],
        *,
        check: Optional[Callable[[Relationship], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Relationship]: ...
    @overload
    def wait_for(
        self,
        event: Literal['relationship_remove'],
        *,
        check: Optional[Callable[[Relationship], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Relationship]: ...
    @overload
    def wait_for(
        self,
        event: Literal['relationship_update'],
        *,
        check: Optional[Callable[[Relationship, Relationship], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Tuple[Relationship, Relationship]]: ...
    @overload
    def wait_for(
        self,
        event: Literal['invite_create'],
        *,
        check: Optional[Callable[[Invite], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Invite]: ...
    @overload
    def wait_for(
        self,
        event: Literal['invite_delete'],
        *,
        check: Optional[Callable[[Invite], bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Invite]: ...
    @overload
    def wait_for(
        self,
        event: str,
        *,
        check: Optional[Callable[..., bool]] = ...,
        timeout: Optional[float] = ...,
    ) -> asyncio.Future[Any]: ...
    def event(self, coro: _F) -> _F: ...
    async def change_presence(
        self,
        *,
        activity: Optional[BaseActivity] = ...,
        status: Optional[Status] = ...,
        afk: bool = ...,
    ) -> None: ...
    def fetch_guilds(
        self,
        *,
        limit: int = ...,
        before: Optional[Union[Snowflake, datetime]] = ...,
        after: Optional[Union[Snowflake, datetime]] = ...,
    ) -> GuildIterator: ...
    async def fetch_template(self, code: Union[str, Template]) -> Template: ...
    async def fetch_guild(self, guild_id: int) -> Guild: ...
    async def create_guild(
        self,
        name: str,
        region: Optional[VoiceRegion] = ...,
        icon: Optional[bytes] = ...,
        *,
        code: Optional[str] = ...,
    ) -> Guild: ...
    async def fetch_invite(
        self, url: Union[str, Invite], *, with_counts: bool = ...
    ) -> Invite: ...
    async def delete_invite(self, invite: Union[Invite, str]) -> None: ...
    async def fetch_widget(self, guild_id: int) -> Widget: ...
    async def application_info(self) -> AppInfo: ...
    async def fetch_user(self, user_id: int) -> User: ...
    async def fetch_user_profile(self, user_id: int) -> Profile: ...
    async def fetch_channel(
        self, channel_id: int
    ) -> Union[_GuildChannel, _PrivateChannel]: ...
    async def fetch_webhook(self, webhook_id: int) -> _AsyncWebhook: ...



from typing import Any, Dict, Optional, Union

import aiohttp

class DiscordException(Exception): ...
class ClientException(DiscordException): ...
class NoMoreItems(DiscordException): ...
class GatewayNotFound(DiscordException): ...

def flatten_error_dict(d: Dict[str, Any], key: str = ...) -> Dict[str, Any]: ...

class HTTPException(DiscordException):
    response: aiohttp.ClientResponse
    text: str
    status: int
    code: int
    def __init__(
        self, response: aiohttp.ClientResponse, message: Union[str, Dict[str, Any]]
    ) -> None: ...

class Forbidden(HTTPException): ...
class NotFound(HTTPException): ...
class DiscordServerError(HTTPException): ...
class InvalidData(ClientException): ...
class InvalidArgument(ClientException): ...
class LoginFailure(ClientException): ...

class ConnectionClosed(ClientException):
    code: Optional[int]
    reason: str
    shard_id: Optional[int]
    def __init__(
        self,
        socket: aiohttp.ClientWebSocketResponse,
        *,
        shard_id: Optional[int],
        code: Optional[int] = ...,
    ) -> None: ...

class PrivilegedIntentsRequired(ClientException):
    shard_id: Optional[int]



import datetime
from typing import Any, List, NamedTuple, Optional, type_check_only

import discord.abc

from .asset import _VALID_ANIMATED_ICON_FORMATS, _VALID_STATIC_ICON_FORMATS, Asset
from .channel import DMChannel, GroupChannel
from .colour import Colour
from .enums import (
    DefaultAvatar,
    FriendFlags,
    HypeSquadHouse,
    PremiumType,
    Status,
    Theme,
    UserContentFilter,
)
from .flags import PublicUserFlags
from .guild import Guild
from .http import _ClientUserDict
from .message import Message
from .permissions import Permissions
from .relationship import Relationship

class Profile(NamedTuple):
    flags: int
    user: User
    mutual_guilds: List[Guild]
    connected_accounts: List[Any]
    premium_since: Optional[datetime.datetime]
    @property
    def nitro(self) -> bool: ...
    @property
    def premium(self) -> bool: ...
    @property
    def staff(self) -> bool: ...
    @property
    def partner(self) -> bool: ...
    @property
    def bug_hunter(self) -> bool: ...
    @property
    def early_supporter(self) -> bool: ...
    @property
    def hypesquad(self) -> bool: ...
    @property
    def hypesquad_houses(self) -> List[HypeSquadHouse]: ...
    @property
    def team_user(self) -> bool: ...
    @property
    def system(self) -> bool: ...

# The following is only used in the stubs in order to allow Member to use the
# properties of BaseUser without inheriting from it and creating a false
# positive between BaseUser and Member
@type_check_only
class _CommonUser:
    name: str
    id: int
    discriminator: str
    avatar: Optional[str]
    bot: bool
    system: bool
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def public_flags(self) -> PublicUserFlags: ...
    @property
    def avatar_url(self) -> Asset: ...
    def is_avatar_animated(self) -> bool: ...
    def avatar_url_as(
        self,
        *,
        format: Optional[_VALID_ANIMATED_ICON_FORMATS] = ...,
        static_format: _VALID_STATIC_ICON_FORMATS = ...,
        size: int = ...,
    ) -> Asset: ...
    @property
    def default_avatar(self) -> DefaultAvatar: ...
    @property
    def default_avatar_url(self) -> Asset: ...
    @property
    def colour(self) -> Colour: ...
    @property
    def color(self) -> Colour: ...
    @property
    def mention(self) -> str: ...
    def permissions_in(self, channel: discord.abc.GuildChannel) -> Permissions: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def display_name(self) -> str: ...
    def mentioned_in(self, message: Message) -> bool: ...

class BaseUser(_CommonUser, discord.abc.User): ...

class ClientUser(BaseUser):
    verified: bool
    email: Optional[str]
    locale: Optional[str]
    mfa_enabled: bool
    premium: bool
    premium_type: Optional[PremiumType]
    def get_relationship(self, user_id: int) -> Optional[Relationship]: ...
    @property
    def relationships(self) -> List[Relationship]: ...
    @property
    def friends(self) -> List[User]: ...
    @property
    def blocked(self) -> List[User]: ...
    async def edit(
        self,
        *,
        password: str = ...,
        new_password: str = ...,
        email: str = ...,
        username: str = ...,
        avatar: bytes = ...,
    ) -> None: ...
    async def create_group(self, *recipients: User) -> GroupChannel: ...
    async def edit_settings(
        self,
        *,
        afk_timeout: int = ...,
        animate_emojis: bool = ...,
        convert_emoticons: bool = ...,
        default_guilds_restricted: bool = ...,
        detect_platform_accounts: bool = ...,
        developer_mode: bool = ...,
        disable_games_tab: bool = ...,
        enable_tts_command: bool = ...,
        explicit_content_filter: UserContentFilter = ...,
        friend_source_flags: FriendFlags = ...,
        gif_auto_play: bool = ...,
        guild_positions: List[discord.abc.Snowflake] = ...,
        inline_attachment_media: bool = ...,
        inline_embed_media: bool = ...,
        locale: str = ...,
        message_display_compact: bool = ...,
        render_embeds: bool = ...,
        render_reactions: bool = ...,
        restricted_guilds: List[discord.abc.Snowflake] = ...,
        show_current_game: bool = ...,
        status: Status = ...,
        theme: Theme = ...,
        timezone_offset: int = ...,
    ) -> _ClientUserDict: ...

# The following is only used in the stubs in order to allow Member to use the
# properties of User without inheriting from it and creating a false
# positive between User and Member
@type_check_only
class _User:
    @property
    def dm_channel(self) -> Optional[DMChannel]: ...
    @property
    def mutual_guilds(self) -> List[Guild]: ...
    async def create_dm(self) -> DMChannel: ...
    @property
    def relationship(self) -> Optional[Relationship]: ...
    async def mutual_friends(self) -> List[User]: ...
    def is_friend(self) -> bool: ...
    def is_blocked(self) -> bool: ...
    async def block(self) -> None: ...
    async def unblock(self) -> None: ...
    async def remove_friend(self) -> None: ...
    async def send_friend_request(self) -> None: ...
    async def profile(self) -> Profile: ...

class User(_User, BaseUser, discord.abc.Messageable): ...



from datetime import datetime
from typing import List, Optional

from .asset import Asset
from .enums import StickerType
from .mixins import Hashable

class Sticker(Hashable):
    id: int
    name: str
    description: str
    pack_id: int
    format: StickerType
    image: str
    tags: List[str]
    preview_image: Optional[str]
    @property
    def created_at(self) -> datetime: ...
    @property
    def image_url(self) -> Asset: ...
    def image_url_as(self, *, size: int = ...) -> Optional[Asset]: ...



import asyncio
from types import MappingProxyType, ModuleType
from typing import (
    Any,
    Callable,
    Collection,
    Coroutine,
    Dict,
    Iterable,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

import aiohttp

import discord

from .cog import Cog
from .context import Context
from .core import GroupMixin
from .help import HelpCommand

_CT = TypeVar('_CT', bound=Context)
_OT = TypeVar('_OT', bound=Context)

_CoroType = Callable[..., Coroutine[Any, Any, Any]]
_C = TypeVar('_C', bound=_CoroType)

_T = TypeVar('_T')
_CallablePrefix = Callable[[Bot, discord.Message], Union[_T, Coroutine[Any, Any, _T]]]
_CommandPrefix = Union[
    str, Iterable[str], _CallablePrefix[str], _CallablePrefix[Iterable[str]]
]

def when_mentioned(bot: Bot[Any], msg: discord.Message) -> List[str]: ...
def when_mentioned_or(
    *prefixes: str,
) -> Callable[[Bot[Any], discord.Message], List[str]]: ...

class BotBase(GroupMixin[_CT]):
    command_prefix: _CommandPrefix
    extra_events: Dict[str, List[Callable[..., Coroutine[Any, Any, None]]]]
    case_insensitive: bool
    description: str
    owner_id: Optional[int]
    owner_ids: Optional[Collection[int]]
    strip_after_prefix: bool
    help_command: Optional[HelpCommand[_CT]]
    def __init__(
        self,
        command_prefix: _CommandPrefix,
        help_command: Optional[HelpCommand[_CT]] = ...,
        description: Optional[str] = ...,
        **options: Any,
    ) -> None: ...
    def dispatch(self, __event_name: str, *args: Any, **kwargs: Any) -> None: ...
    async def close(self) -> None: ...
    async def on_command_error(self, context: _CT, exception: Exception) -> None: ...
    def check(self, func: _C) -> _C: ...
    def add_check(self, func: _CoroType, *, call_once: bool = ...) -> None: ...
    def remove_check(self, func: _CoroType, *, call_once: bool = ...) -> None: ...
    def check_once(self, func: _C) -> _C: ...
    async def can_run(self, ctx: _CT, *, call_once: bool = ...) -> bool: ...
    async def is_owner(self, user: Union[discord.User, discord.Member]) -> bool: ...
    def before_invoke(self, coro: _C) -> _C: ...
    def after_invoke(self, coro: _C) -> _C: ...
    def add_listener(self, func: _CoroType, name: Optional[str] = ...) -> None: ...
    def remove_listener(self, func: _CoroType, name: Optional[str] = ...) -> None: ...
    def listen(self, name: Optional[str] = ...) -> Callable[[_C], _C]: ...
    def add_cog(self, cog: Cog[_CT]) -> None: ...
    def get_cog(self, name: str) -> Optional[Cog[_CT]]: ...
    def remove_cog(self, name: str) -> None: ...
    @property
    def cogs(self) -> MappingProxyType[str, Cog[_CT]]: ...
    def load_extension(self, name: str, *, package: Optional[str] = ...) -> None: ...
    def unload_extension(self, name: str, *, package: Optional[str] = ...) -> None: ...
    def reload_extension(self, name: str, *, package: Optional[str] = ...) -> None: ...
    @property
    def extensions(self) -> MappingProxyType[str, ModuleType]: ...
    async def get_prefix(self, message: discord.Message) -> Union[List[str], str]: ...
    @overload
    async def get_context(self, message: discord.Message) -> _CT: ...
    @overload
    async def get_context(self, message: discord.Message, *, cls: Type[_OT]) -> _OT: ...
    async def invoke(self, ctx: _CT) -> None: ...
    async def process_commands(self, message: discord.Message) -> None: ...
    async def on_message(self, message: discord.Message) -> None: ...

class Bot(BotBase[_CT], discord.Client):
    @overload
    def __init__(
        self: Bot[Context],
        command_prefix: _CommandPrefix,
        help_command: Optional[HelpCommand[Context]] = ...,
        description: Optional[str] = ...,
        *,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        owner_id: int,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        shard_id: Optional[int] = ...,
        shard_count: Optional[int] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: discord.MemberCacheFlags = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: Bot[Context],
        command_prefix: _CommandPrefix,
        help_command: Optional[HelpCommand[Context]] = ...,
        description: Optional[str] = ...,
        *,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        owner_ids: Collection[int],
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        shard_id: Optional[int] = ...,
        shard_count: Optional[int] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: discord.MemberCacheFlags = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: Bot[Context],
        command_prefix: _CommandPrefix,
        help_command: Optional[HelpCommand[Context]] = ...,
        description: Optional[str] = ...,
        *,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        shard_id: Optional[int] = ...,
        shard_count: Optional[int] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: discord.MemberCacheFlags = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...

class AutoShardedBot(BotBase[_CT], discord.AutoShardedClient):
    @overload
    def __init__(
        self: AutoShardedBot[Context],
        command_prefix: _CommandPrefix,
        help_command: HelpCommand[Context] = ...,
        description: Optional[str] = ...,
        *,
        owner_id: int,
        shard_ids: Union[List[int], Tuple[int]],
        shard_count: int,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: Optional[discord.MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: AutoShardedBot[Context],
        command_prefix: _CommandPrefix,
        help_command: HelpCommand[Context] = ...,
        description: Optional[str] = ...,
        *,
        owner_ids: Collection[int],
        shard_ids: Union[List[int], Tuple[int]],
        shard_count: int,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: Optional[discord.MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: AutoShardedBot[Context],
        command_prefix: _CommandPrefix,
        help_command: HelpCommand[Context] = ...,
        description: Optional[str] = ...,
        *,
        shard_ids: Union[List[int], Tuple[int]],
        shard_count: int,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: Optional[discord.MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: AutoShardedBot[Context],
        command_prefix: _CommandPrefix,
        help_command: HelpCommand[Context] = ...,
        description: Optional[str] = ...,
        *,
        owner_id: int,
        shard_ids: None = ...,
        shard_count: Optional[int] = ...,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: Optional[discord.MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: AutoShardedBot[Context],
        command_prefix: _CommandPrefix,
        help_command: HelpCommand[Context] = ...,
        description: Optional[str] = ...,
        *,
        owner_ids: Collection[int],
        shard_ids: None = ...,
        shard_count: Optional[int] = ...,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: Optional[discord.MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: AutoShardedBot[Context],
        command_prefix: _CommandPrefix,
        help_command: HelpCommand[Context] = ...,
        description: Optional[str] = ...,
        *,
        shard_ids: None = ...,
        shard_count: Optional[int] = ...,
        case_insensitive: bool = ...,
        strip_after_prefix: bool = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[discord.Intents] = ...,
        member_cache_flags: Optional[discord.MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[discord.Status] = ...,
        activity: Optional[
            Union[discord.Activity, discord.Game, discord.Streaming]
        ] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...



import datetime
from typing import Any, List, Optional, Tuple, Union

import discord.abc

from .activity import BaseActivity, Spotify
from .channel import GroupChannel, StageChannel, VoiceChannel
from .colour import Colour
from .enums import Status
from .guild import Guild
from .message import Message
from .permissions import Permissions
from .role import Role
from .user import _CommonUser, _User

class VoiceState:
    deaf: bool
    mute: bool
    self_mute: bool
    self_deaf: bool
    self_stream: bool
    self_video: bool
    suppress: bool
    requested_to_speak_at: Optional[datetime.datetime]
    afk: bool
    channel: Optional[Union[GroupChannel, VoiceChannel, StageChannel]]
    session_id: str

class Member(_CommonUser, _User, discord.abc.Messageable, discord.abc.User):
    joined_at: Optional[datetime.datetime]
    activities: Tuple[Union[BaseActivity, Spotify], ...]
    nick: Optional[str]
    pending: bool
    premium_since: Optional[datetime.datetime]
    guild: Guild
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def status(self) -> Union[Status, str]: ...
    @property
    def raw_status(self) -> str: ...
    @property
    def mobile_status(self) -> Status: ...
    @property
    def desktop_status(self) -> Status: ...
    @property
    def web_status(self) -> Status: ...
    def is_on_mobile(self) -> bool: ...
    @property
    def colour(self) -> Colour: ...
    @property
    def color(self) -> Colour: ...
    @property
    def roles(self) -> List[Role]: ...
    @property
    def mention(self) -> str: ...
    @property
    def display_name(self) -> str: ...
    @property
    def activity(self) -> Union[BaseActivity, Spotify]: ...
    def mentioned_in(self, message: Message) -> bool: ...
    def permissions_in(self, channel: discord.abc.GuildChannel) -> Permissions: ...
    @property
    def top_role(self) -> Role: ...
    @property
    def guild_permissions(self) -> Permissions: ...
    @property
    def voice(self) -> Optional[VoiceState]: ...
    async def ban(
        self, *, reason: Optional[str] = ..., delete_message_days: int = ...
    ) -> None: ...
    async def unban(self, *, reason: Optional[str] = ...) -> None: ...
    async def kick(self, *, reason: Optional[str] = ...) -> None: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        nick: Optional[str] = ...,
        mute: bool = ...,
        deafen: bool = ...,
        suppress: bool = ...,
        roles: List[Role] = ...,
        voice_channel: Optional[VoiceChannel] = ...,
    ) -> None: ...
    async def request_to_speak(self) -> None: ...
    async def move_to(
        self, channel: Optional[VoiceChannel], *, reason: Optional[str] = ...
    ) -> None: ...
    async def add_roles(
        self,
        *roles: discord.abc.Snowflake,
        reason: Optional[str] = ...,
        atomic: bool = ...,
    ) -> None: ...
    async def remove_roles(
        self,
        *roles: discord.abc.Snowflake,
        reason: Optional[str] = ...,
        atomic: bool = ...,
    ) -> None: ...



from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import Any, Tuple

def show_version() -> None: ...
def core(parser: Any, args: Any) -> None: ...

bot_template: str
gitignore_template: str
cog_template: str
cog_extras: str
translation_table: Any

def to_path(parser: Any, name: Any, *, replace_spaces: bool = ...) -> Path: ...
def newbot(parser: Any, args: Any) -> None: ...
def newcog(parser: Any, args: Any) -> None: ...
def add_newbot_args(subparser: Any) -> None: ...
def add_newcog_args(subparser: Any) -> None: ...
def parse_args() -> Tuple[ArgumentParser, Namespace]: ...
def main() -> None: ...



from typing import NamedTuple
from typing_extensions import Final

from . import abc as abc  # noqa: F401
from . import opus as opus  # noqa: F401
from . import utils as utils  # noqa: F401
from .activity import *  # noqa: F401
from .appinfo import AppInfo as AppInfo  # noqa: F401
from .asset import Asset as Asset  # noqa: F401
from .audit_logs import AuditLogChanges as AuditLogChanges  # noqa: F401
from .audit_logs import AuditLogDiff as AuditLogDiff  # noqa: F401
from .audit_logs import AuditLogEntry as AuditLogEntry  # noqa: F401
from .calls import CallMessage as CallMessage  # noqa: F401
from .calls import GroupCall as GroupCall  # noqa: F401
from .channel import CategoryChannel as CategoryChannel  # noqa: F401
from .channel import DMChannel as DMChannel  # noqa: F401
from .channel import GroupChannel as GroupChannel  # noqa: F401
from .channel import StageChannel as StageChannel  # noqa: F401
from .channel import StoreChannel as StoreChannel  # noqa: F401
from .channel import TextChannel as TextChannel  # noqa: F401
from .channel import VoiceChannel as VoiceChannel  # noqa: F401
from .client import Client as Client  # noqa: F401
from .colour import Color as Color  # noqa: F401
from .colour import Colour as Colour  # noqa: F401
from .embeds import Embed as Embed  # noqa: F401
from .emoji import Emoji as Emoji  # noqa: F401
from .enums import *  # noqa: F401
from .errors import *  # noqa: F401
from .file import File as File  # noqa: F401
from .flags import Intents as Intents  # noqa: F401
from .flags import MemberCacheFlags as MemberCacheFlags  # noqa: F401
from .flags import MessageFlags as MessageFlags  # noqa: F401
from .flags import PublicUserFlags as PublicUserFlags  # noqa: F401
from .flags import SystemChannelFlags as SystemChannelFlags  # noqa: F401
from .guild import Guild as Guild  # noqa: F401
from .integrations import Integration as Integration  # noqa: F401
from .integrations import IntegrationAccount as IntegrationAccount  # noqa: F401
from .invite import Invite as Invite  # noqa: F401
from .member import Member as Member  # noqa: F401
from .member import VoiceState as VoiceState  # noqa: F401
from .mentions import AllowedMentions as AllowedMentions  # noqa: F401
from .message import Attachment as Attachment  # noqa: F401
from .message import DeletedReferencedMessage as DeletedReferencedMessage  # noqa: F401
from .message import Message as Message  # noqa: F401
from .message import MessageReference as MessageReference  # noqa: F401
from .message import PartialMessage as PartialMessage  # noqa: F401
from .object import Object as Object  # noqa: F401
from .partial_emoji import PartialEmoji as PartialEmoji  # noqa: F401
from .permissions import PermissionOverwrite as PermissionOverwrite  # noqa: F401
from .permissions import Permissions as Permissions  # noqa: F401
from .player import AudioSource as AudioSource  # noqa: F401
from .player import FFmpegAudio as FFmpegAudio  # noqa: F401
from .player import FFmpegOpusAudio as FFmpegOpusAudio  # noqa: F401
from .player import FFmpegPCMAudio as FFmpegPCMAudio  # noqa: F401
from .player import PCMAudio as PCMAudio  # noqa: F401
from .player import PCMVolumeTransformer as PCMVolumeTransformer  # noqa: F401
from .raw_models import *  # noqa: F401
from .reaction import Reaction as Reaction  # noqa: F401
from .relationship import Relationship as Relationship  # noqa: F401
from .role import Role as Role  # noqa: F401
from .role import RoleTags as RoleTags  # noqa: F401
from .shard import AutoShardedClient as AutoShardedClient  # noqa: F401
from .shard import ShardInfo as ShardInfo  # noqa: F401
from .sticker import Sticker as Sticker  # noqa: F401
from .team import *  # noqa: F401
from .template import Template as Template  # noqa: F401
from .user import ClientUser as ClientUser  # noqa: F401
from .user import Profile as Profile  # noqa: F401
from .user import User as User  # noqa: F401
from .voice_client import VoiceClient as VoiceClient  # noqa: F401
from .voice_client import VoiceProtocol as VoiceProtocol  # noqa: F401
from .webhook import *  # noqa: F401
from .widget import Widget as Widget  # noqa: F401
from .widget import WidgetChannel as WidgetChannel  # noqa: F401
from .widget import WidgetMember as WidgetMember  # noqa: F401

class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: int

__title__: Final[str]
__author__: Final[str]
__license__: Final[str]
__copyright__: Final[str]
__version__: Final[str]
version_info: Final[VersionInfo]



from typing import List, Optional

from .asset import _VALID_STATIC_ICON_FORMATS, Asset
from .enums import TeamMembershipState
from .user import BaseUser

class Team:
    id: int
    name: str
    icon: Optional[str]
    owner_id: int
    members: List[TeamMember]
    @property
    def icon_url(self) -> Asset: ...
    def icon_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def owner(self) -> Optional[TeamMember]: ...

class TeamMember(BaseUser):
    team: Team
    membership_state: TeamMembershipState
    permissions: List[str]



from typing import (
    Any,
    Callable,
    ClassVar,
    Dict,
    Iterator,
    List,
    Tuple,
    Type,
    TypeVar,
    overload,
)

from .enums import UserFlags

_F = TypeVar('_F', bound=flag_value)
_I = TypeVar('_I', bound=Intents)
_MCF = TypeVar('_MCF', bound=MemberCacheFlags)

class flag_value:
    flag: int
    def __init__(self, func: Callable[[Any], int]) -> None: ...
    @overload
    def __get__(self: _F, instance: None, owner: Any) -> _F: ...
    @overload
    def __get__(self, instance: Any, owner: Any) -> bool: ...
    def __set__(self, instance: Any, value: bool) -> None: ...

class alias_flag_value(flag_value): ...

class BaseFlags:
    value: int
    def __init__(self, **kwargs: bool) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __iter__(self) -> Iterator[Tuple[str, bool]]: ...

class SystemChannelFlags(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]

    join_notifications: flag_value
    premium_subscriptions: flag_value

class MessageFlags(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]

    crossposted: flag_value
    is_crossposted: flag_value
    suppress_embeds: flag_value
    source_message_deleted: flag_value
    urgent: flag_value

class PublicUserFlags(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]

    staff: flag_value
    partner: flag_value
    hypesquad: flag_value
    bug_hunter: flag_value
    hypesquad_bravery: flag_value
    hypesquad_brilliance: flag_value
    hypesquad_balance: flag_value
    early_supporter: flag_value
    team_user: flag_value
    system: flag_value
    bug_hunter_level_2: flag_value
    verified_bot: flag_value
    verified_bot_developer: flag_value
    early_verified_bot_developer: alias_flag_value
    def all(self) -> List[UserFlags]: ...

class Intents(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]
    def __init__(
        self,
        *,
        guilds: bool = ...,
        members: bool = ...,
        bans: bool = ...,
        emojis: bool = ...,
        integrations: bool = ...,
        webhooks: bool = ...,
        invites: bool = ...,
        voice_states: bool = ...,
        presences: bool = ...,
        messages: bool = ...,
        guild_messages: bool = ...,
        dm_messages: bool = ...,
        reactions: bool = ...,
        guild_reactions: bool = ...,
        dm_reactions: bool = ...,
        typing: bool = ...,
        guild_typing: bool = ...,
        dm_typing: bool = ...,
    ) -> None: ...
    @classmethod
    def all(cls: Type[_I]) -> _I: ...
    @classmethod
    def none(cls: Type[_I]) -> _I: ...
    @classmethod
    def default(cls: Type[_I]) -> _I: ...
    guilds: flag_value
    members: flag_value
    bans: flag_value
    emojis: flag_value
    integrations: flag_value
    webhooks: flag_value
    invites: flag_value
    voice_states: flag_value
    presences: flag_value
    messages: alias_flag_value
    guild_messages: flag_value
    dm_messages: flag_value
    reactions: alias_flag_value
    guild_reactions: flag_value
    dm_reactions: flag_value
    typing: alias_flag_value
    guild_typing: flag_value
    dm_typing: flag_value

class MemberCacheFlags(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]
    def __init__(
        self, *, online: bool = ..., voice: bool = ..., joined: bool = ...
    ) -> None: ...
    @classmethod
    def all(cls: Type[_MCF]) -> _MCF: ...
    @classmethod
    def none(cls: Type[_MCF]) -> _MCF: ...
    online: flag_value
    voice: flag_value
    joined: flag_value
    @classmethod
    def from_intents(cls: Type[_MCF], intents: Intents) -> _MCF: ...



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['discord-stubs']

package_data = \
{'': ['*'], 'discord-stubs': ['ext/commands/*', 'ext/tasks/*']}

install_requires = \
['discord.py>=1.7.0,<1.8.0', 'mypy>=0.800', 'typing-extensions>=3.7.4,<4.0.0']

setup_kwargs = {
    'name': 'discord.py-stubs',
    'version': '1.7.0',
    'description': 'discord.py stubs',
    'long_description': '# discord.py-stubs\n\n[![License](https://img.shields.io/badge/License-BSD%203--Clause-blue.svg)](https://github.com/bryanforbes/discord.py-stubs/blob/master/LICENSE)\n[![Checked with mypy](http://www.mypy-lang.org/static/mypy_badge.svg)](http://mypy-lang.org/)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n\nThis package contains type stubs to provide more precise static types and type inference for discord.py.\n\n## Installation\n\n```\npip install discord.py-stubs\n```\n\n**NOTE:** Because `discord.py` uses namespace packages for its extensions, `mypy` must be configured to use namespace packages either with the `--namespace-packages` command line flag, or by setting `namespace_packages = True` in your `mypy` configuration file. See the [import discovery](https://mypy.readthedocs.io/en/stable/command_line.html#import-discovery) section of the `mypy` documentation for more details.\n\n## Usage Notes\n\nIn most cases, installing this package will enable developers to type check their discord.py bots using mypy out of the box. However, if developers wish to subclass the classes in `discord.ext.commands` they will need to follow the `mypy` documentation outlining how to use [classes that are generic in stubs but not at runtime](https://mypy.readthedocs.io/en/stable/common_issues.html#using-classes-that-are-generic-in-stubs-but-not-at-runtime):\n\n```python\nfrom typing import TYPE_CHECKING\nfrom discord.ext import commands\n\nclass MyContext(commands.Context):\n    ...\n\nif TYPE_CHECKING:\n    Cog = commands.Cog[MyContext]\nelse:\n    Cog = commands.Cog\n\nclass MyCog(Cog):\n    ...\n```\n\nIn order to avoid this issue, developers can use [`discord-ext-typed-commands`](https://github.com/bryanforbes/discord-ext-typed-commands/):\n\n```python\nfrom discord.ext import typed_commands\n\nclass MyContext(typed_commands.Context):\n    ...\n\nclass MyCog(typed_commands.Cog[MyContext]):\n    ...\n```\n\n## Development\n\nMake sure you have [poetry](https://python-poetry.org/) installed.\n\n```\npoetry install\npoetry run pre-commit install --hook-type pre-commit --hook-type post-checkout\n```\n\n\n## Version numbering scheme\n\nThe **major** and **minor** version numbers of `discord.py-stubs` will match the **major** and **minor** version numbers of the `discord.py` release the stubs represent. For instance, if you are using `discord.py` version `1.7.4`, you would use `discord.py-stubs` version `1.7.X` where `X` is the latest **patch** version of the stubs. Using semver dependency specifications, `discord.py-stubs` version `~1.7` is designed to work with `discord.py` version `~1.7`.\n\nIn addition, `discord.py-stubs` will indicate which versions of the runtime library are compatible through its dependency information (as suggested in PEP-561).\n',
    'author': 'Bryan Forbes',
    'author_email': 'bryan@reigndropsfall.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bryanforbes/discord.py-stubs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



import datetime
from typing import Callable, Dict, Iterable, List, Optional, Union, overload

import discord.abc

from .asset import _VALID_STATIC_ICON_FORMATS, Asset
from .enums import ChannelType, VoiceRegion
from .member import Member, VoiceState
from .message import Message, PartialMessage
from .mixins import Hashable
from .permissions import PermissionOverwrite, Permissions
from .role import Role
from .user import BaseUser, ClientUser, User
from .webhook import _AsyncWebhook

_OverwritesDict = Dict[Union[Role, Member], PermissionOverwrite]

class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
    topic: Optional[str]
    last_message_id: Optional[int]
    slowmode_delay: int
    nsfw: bool
    @property
    def type(self) -> ChannelType: ...
    def permissions_for(self, member: Member) -> Permissions: ...
    @property
    def members(self) -> List[Member]: ...
    def is_nsfw(self) -> bool: ...
    def is_news(self) -> bool: ...
    @property
    def last_message(self) -> Optional[Message]: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        name: str = ...,
        topic: str = ...,
        position: int = ...,
        nsfw: bool = ...,
        sync_permissions: bool = ...,
        category: Optional[CategoryChannel] = ...,
        slowmode_delay: int = ...,
        type: ChannelType = ...,
        overwrites: _OverwritesDict = ...,
    ) -> None: ...
    async def delete_messages(self, messages: Iterable[Message]) -> None: ...
    async def purge(
        self,
        *,
        limit: Optional[int] = ...,
        check: Optional[Callable[[Message], bool]] = ...,
        before: Optional[Union[datetime.datetime, Message]] = ...,
        after: Optional[Union[datetime.datetime, Message]] = ...,
        around: Optional[Union[datetime.datetime, Message]] = ...,
        oldest_first: Optional[bool] = ...,
        bulk: bool = ...,
    ) -> List[Message]: ...
    async def webhooks(self) -> List[_AsyncWebhook]: ...
    async def create_webhook(
        self,
        *,
        name: str,
        avatar: Optional[Union[bytes, bytearray]] = ...,
        reason: Optional[str] = ...,
    ) -> _AsyncWebhook: ...
    async def follow(
        self, *, destination: TextChannel, reason: Optional[str] = ...
    ) -> _AsyncWebhook: ...
    def get_partial_message(self, message_id: int) -> PartialMessage: ...

class VocalGuildChannel(discord.abc.Connectable, discord.abc.GuildChannel, Hashable):
    bitrate: int
    user_limit: int
    rtc_region: Optional[VoiceRegion]
    @property
    def members(self) -> List[Member]: ...
    @property
    def voice_states(self) -> Dict[int, VoiceState]: ...
    def permissions_for(self, member: Member) -> Permissions: ...

class VoiceChannel(VocalGuildChannel):
    @property
    def type(self) -> ChannelType: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        name: str = ...,
        bitrate: int = ...,
        user_limit: int = ...,
        position: int = ...,
        sync_permissions: bool = ...,
        category: Optional[CategoryChannel] = ...,
        overwrites: _OverwritesDict = ...,
        rtc_region: Optional[VoiceRegion] = ...,
    ) -> None: ...

class StageChannel(VocalGuildChannel):
    topic: str
    @property
    def requesting_to_speak(self) -> List[Member]: ...
    @property
    def type(self) -> ChannelType: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        name: str = ...,
        topic: str = ...,
        position: int = ...,
        sync_permissions: bool = ...,
        category: Optional[CategoryChannel] = ...,
        overwrites: _OverwritesDict = ...,
        rtc_region: Optional[VoiceRegion] = ...,
    ) -> None: ...

class CategoryChannel(discord.abc.GuildChannel, Hashable):
    nsfw: bool
    @property
    def type(self) -> ChannelType: ...
    def is_nsfw(self) -> bool: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        name: str = ...,
        position: int = ...,
        nsfw: bool = ...,
        overwrites: Dict[Union[Role, Member], PermissionOverwrite] = ...,
    ) -> None: ...
    @overload
    async def move(
        self,
        *,
        beginning: bool,
        offset: int = ...,
        category: Optional[discord.abc.Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    @overload
    async def move(
        self,
        *,
        end: bool,
        offset: int = ...,
        category: Optional[discord.abc.Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    @overload
    async def move(
        self,
        *,
        before: discord.abc.Snowflake,
        offset: int = ...,
        category: Optional[discord.abc.Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    @overload
    async def move(
        self,
        *,
        after: discord.abc.Snowflake,
        offset: int = ...,
        category: Optional[discord.abc.Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    @property
    def channels(self) -> List[Union[TextChannel, VoiceChannel, StoreChannel]]: ...
    @property
    def text_channels(self) -> List[TextChannel]: ...
    @property
    def voice_channels(self) -> List[VoiceChannel]: ...
    @property
    def stage_channels(self) -> List[StageChannel]: ...
    async def create_text_channel(
        self,
        name: str,
        *,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        position: int = ...,
        topic: str = ...,
        slowmode_delay: int = ...,
        nsfw: bool = ...,
        reason: Optional[str] = ...,
    ) -> TextChannel: ...
    async def create_voice_channel(
        self,
        name: str,
        *,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        bitrate: int = ...,
        position: int = ...,
        user_limit: int = ...,
        rtc_region: Optional[VoiceRegion] = ...,
        reason: Optional[str] = ...,
    ) -> VoiceChannel: ...
    async def create_stage_channel(
        self,
        name: str,
        *,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        topic: str = ...,
        position: int = ...,
        rtc_region: Optional[VoiceRegion] = ...,
        reason: Optional[str] = ...,
    ) -> StageChannel: ...

class StoreChannel(discord.abc.GuildChannel, Hashable):
    nsfw: bool
    @property
    def type(self) -> ChannelType: ...
    def permissions_for(self, member: Member) -> Permissions: ...
    def is_nsfw(self) -> bool: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        name: str = ...,
        position: int = ...,
        nsfw: bool = ...,
        sync_permissions: bool = ...,
        category: Optional[CategoryChannel] = ...,
        overwrites: _OverwritesDict = ...,
    ) -> None: ...

class DMChannel(discord.abc.Messageable, Hashable):
    id: int
    recipient: User
    me: ClientUser
    @property
    def type(self) -> ChannelType: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    def permissions_for(self, user: Optional[BaseUser] = ...) -> Permissions: ...
    def get_partial_message(self, message_id: int) -> PartialMessage: ...

class GroupChannel(discord.abc.Messageable, Hashable):
    id: int
    me: ClientUser
    owner: User
    icon: Optional[str]
    name: Optional[str]
    recipients: List[User]
    @property
    def type(self) -> ChannelType: ...
    @property
    def icon_url(self) -> Asset: ...
    def icon_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    def permissions_for(self, user: BaseUser) -> Permissions: ...
    async def add_recipients(self, *recipients: User) -> None: ...
    async def remove_recipients(self, *recipients: User) -> None: ...
    async def edit(
        self, name: Optional[str] = ..., icon: Optional[bytes] = ...
    ) -> None: ...
    async def leave(self) -> None: ...



from ctypes import Structure, c_float, c_int, c_int16, pointer
from typing import Any, ClassVar, Optional, Type, Union, type_check_only
from typing_extensions import Final, TypedDict

from .errors import DiscordException

c_int_ptr: Final[Type[pointer[c_int]]]
c_int16_ptr: Final[Type[pointer[c_int16]]]
c_float_ptr: Final[Type[pointer[c_float]]]

class EncoderStruct(Structure): ...
class DecoderStruct(Structure): ...

EncoderStructPtr: Final[Type[pointer[EncoderStruct]]]
DecoderStructPtr: Final[Type[pointer[DecoderStruct]]]

OK: Final[int]
BAD_ARG: Final[int]
APPLICATION_AUDIO: Final[int]
APPLICATION_VOIP: Final[int]
APPLICATION_LOWDELAY: Final[int]
CTL_SET_BITRATE: Final[int]
CTL_SET_BANDWIDTH: Final[int]
CTL_SET_FEC: Final[int]
CTL_SET_PLP: Final[int]
CTL_SET_SIGNAL: Final[int]
CTL_SET_GAIN: Final[int]
CTL_LAST_PACKET_DURATION: Final[int]

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _BandCtl(TypedDict):
    narrow: int
    medium: int
    wide: int
    superwide: int
    full: int

@type_check_only
class _SignalCtl(TypedDict):
    auto: int
    voice: int
    music: int

band_ctl: Final[_BandCtl]
signal_ctl: Final[_SignalCtl]

def libopus_loader(name: str) -> Any: ...
def load_opus(name: str) -> None: ...
def is_loaded() -> bool: ...

class OpusError(DiscordException):
    code: int

class OpusNotLoaded(DiscordException): ...

class _OpusStruct:
    SAMPLING_RATE: ClassVar[int]
    CHANNELS: ClassVar[int]
    FRAME_LENGTH: ClassVar[int]
    SAMPLE_SIZE: ClassVar[int]
    SAMPLES_PER_FRAME: ClassVar[int]
    FRAME_SIZE: ClassVar[int]
    @staticmethod
    def get_opus_version() -> str: ...

class Encoder(_OpusStruct):
    application: int
    def __init__(self, application: int = ...) -> None: ...
    def __del__(self) -> None: ...
    def set_bitrate(self, kbps: int) -> int: ...
    def set_bandwidth(self, req: str) -> None: ...
    def set_signal_type(self, req: str) -> None: ...
    def set_fec(self, enabled: bool = ...) -> None: ...
    def set_expected_packet_loss_percent(
        self, percentage: Union[int, float]
    ) -> None: ...
    def encode(self, pcm: bytes, frame_size: int) -> bytes: ...

class Decoder(_OpusStruct):
    def __init__(self) -> None: ...
    def __del__(self) -> None: ...
    @staticmethod
    def packet_get_nb_frames(data: bytes) -> int: ...
    @staticmethod
    def packet_get_nb_channels(data: bytes) -> int: ...
    @classmethod
    def packet_get_samples_per_frame(cls, data: bytes) -> int: ...
    def set_gain(self, dB: int) -> int: ...
    def set_volume(self, mult: int) -> int: ...
    def decode(self, data: Optional[bytes], *, fec: bool = ...) -> bytes: ...



from typing import Optional, Tuple, Type, TypeVar, Union
from typing_extensions import Protocol

from discord.enums import Enum
from discord.message import Message

_CM = TypeVar('_CM', bound=CooldownMapping)
_MC = TypeVar('_MC', bound=MaxConcurrency)

_BucketKey = Union[int, Tuple[Optional[int], int]]
_CooldownType = Union[BucketType, _CooldownCallable]

class _CooldownCallable(Protocol):
    def __call__(self, __msg: Message) -> Optional[_BucketKey]: ...

class BucketType(Enum):
    default: int
    user: int
    guild: int
    channel: int
    member: int
    category: int
    role: int
    def get_key(self, msg: Message) -> Optional[_BucketKey]: ...
    def __call__(self, msg: Message) -> Optional[_BucketKey]: ...

class Cooldown:
    rate: int
    per: float
    type: _CooldownType
    def __init__(self, rate: int, per: float, type: _CooldownType) -> None: ...
    def get_tokens(self, current: Optional[int] = ...) -> int: ...
    def get_retry_after(self, current: Optional[float] = ...) -> float: ...
    def update_rate_limit(self, current: Optional[float] = ...) -> Optional[float]: ...
    def reset(self) -> None: ...
    def copy(self) -> Cooldown: ...

class CooldownMapping:
    def __init__(self, original: Cooldown) -> None: ...
    def copy(self) -> CooldownMapping: ...
    @property
    def valid(self) -> bool: ...
    @classmethod
    def from_cooldown(
        cls: Type[_CM],
        rate: int,
        per: float,
        type: _CooldownType,
    ) -> _CM: ...
    def get_bucket(
        self, message: Message, current: Optional[float] = ...
    ) -> Cooldown: ...
    def update_rate_limit(
        self, message: Message, current: Optional[float] = ...
    ) -> Optional[float]: ...

class MaxConcurrency:
    number: int
    per: _CooldownType
    wait: bool
    def __init__(self, number: int, *, per: _CooldownType, wait: bool) -> None: ...
    def copy(self: _MC) -> _MC: ...
    def get_key(self, message: Message) -> Union[str, int]: ...
    async def acquire(self, message: Message) -> None: ...
    async def release(self, message: Message) -> None: ...



# NOTE: ConnectionState should never be accessed by the user and is only included
# here in order to be used in Webhook.from_state(). For these reasons, it is declared
# as an opaque structure

class ConnectionState: ...
class AutoShardedConnectionState(ConnectionState): ...



from datetime import datetime
from typing import Any, List, Optional, Union

from .activity import BaseActivity, Spotify
from .channel import VoiceChannel
from .enums import Status
from .invite import Invite
from .user import BaseUser

class WidgetChannel:
    id: int
    name: str
    position: int
    @property
    def mention(self) -> str: ...
    @property
    def created_at(self) -> datetime: ...

class WidgetMember(BaseUser):
    id: int
    status: Status
    nick: Optional[str]
    activity: Optional[Union[BaseActivity, Spotify]]
    deafened: Optional[bool]
    muted: Optional[bool]
    suppress: Optional[bool]
    connected_channel: Optional[VoiceChannel]
    @property
    def display_name(self) -> str: ...

class Widget:
    id: int
    name: str
    channels: Optional[List[WidgetChannel]]
    members: Optional[List[WidgetMember]]
    def __eq__(self, other: Any) -> bool: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def json_url(self) -> str: ...
    @property
    def invite_url(self) -> Optional[str]: ...
    async def fetch_invite(self, *, with_counts: bool = ...) -> Optional[Invite]: ...



import abc
import datetime
from typing import Dict, List, Optional, Type, TypeVar, Union, overload
from typing_extensions import Protocol, runtime_checkable

from .channel import CategoryChannel
from .context_managers import Typing
from .embeds import Embed
from .file import File
from .guild import Guild
from .invite import Invite
from .iterators import HistoryIterator
from .member import Member
from .mentions import AllowedMentions
from .message import Message, MessageReference
from .permissions import PermissionOverwrite, Permissions
from .role import Role
from .user import ClientUser
from .voice_client import VoiceClient, VoiceProtocol

_GC = TypeVar('_GC', bound=GuildChannel)
_VP = TypeVar('_VP', bound=VoiceProtocol)
@runtime_checkable
class Snowflake(Protocol):
    id: int
    @property
    @abc.abstractmethod
    def created_at(self) -> datetime.datetime: ...

@runtime_checkable
class User(Snowflake, Protocol):
    name: str
    discriminator: str
    avatar: Optional[str]
    bot: bool
    @property
    @abc.abstractmethod
    def display_name(self) -> str: ...
    @property
    @abc.abstractmethod
    def mention(self) -> str: ...

@runtime_checkable
class PrivateChannel(Snowflake, Protocol):
    me: ClientUser

class GuildChannel:
    id: int
    name: str
    guild: Guild
    position: int
    category_id: Optional[int]
    @property
    def changed_roles(self) -> List[Role]: ...
    @property
    def mention(self) -> str: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    def overwrites_for(self, obj: Union[Role, User]) -> PermissionOverwrite: ...
    @property
    def overwrites(self) -> Dict[Union[Role, Member], PermissionOverwrite]: ...
    @property
    def category(self) -> Optional[CategoryChannel]: ...
    @property
    def permissions_synced(self) -> bool: ...
    def permissions_for(self, member: Member) -> Permissions: ...
    async def delete(self, *, reason: Optional[str] = ...) -> None: ...
    @overload
    async def set_permissions(
        self,
        target: Union[Member, Role],
        *,
        overwrite: Optional[PermissionOverwrite] = ...,
        reason: Optional[str] = ...,
    ) -> None: ...
    @overload
    async def set_permissions(
        self,
        target: Union[Member, Role],
        *,
        reason: Optional[str] = ...,
        create_instant_invite: Optional[bool] = ...,
        kick_members: Optional[bool] = ...,
        ban_members: Optional[bool] = ...,
        administrator: Optional[bool] = ...,
        manage_channels: Optional[bool] = ...,
        manage_guild: Optional[bool] = ...,
        add_reactions: Optional[bool] = ...,
        view_audit_log: Optional[bool] = ...,
        priority_speaker: Optional[bool] = ...,
        stream: Optional[bool] = ...,
        read_messages: Optional[bool] = ...,
        view_channel: Optional[bool] = ...,
        send_messages: Optional[bool] = ...,
        send_tts_messages: Optional[bool] = ...,
        manage_messages: Optional[bool] = ...,
        embed_links: Optional[bool] = ...,
        attach_files: Optional[bool] = ...,
        read_message_history: Optional[bool] = ...,
        mention_everyone: Optional[bool] = ...,
        external_emojis: Optional[bool] = ...,
        use_external_emojis: Optional[bool] = ...,
        view_guild_insights: Optional[bool] = ...,
        connect: Optional[bool] = ...,
        speak: Optional[bool] = ...,
        mute_members: Optional[bool] = ...,
        deafen_members: Optional[bool] = ...,
        move_members: Optional[bool] = ...,
        use_voice_activation: Optional[bool] = ...,
        change_nickname: Optional[bool] = ...,
        manage_nicknames: Optional[bool] = ...,
        manage_roles: Optional[bool] = ...,
        manage_permissions: Optional[bool] = ...,
        manage_webhooks: Optional[bool] = ...,
        manage_emojis: Optional[bool] = ...,
    ) -> None: ...
    async def clone(
        self: _GC, *, name: Optional[str] = ..., reason: Optional[str] = ...
    ) -> _GC: ...
    @overload
    async def move(
        self,
        *,
        beginning: bool,
        offset: int = ...,
        category: Optional[Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    @overload
    async def move(
        self,
        *,
        end: bool,
        offset: int = ...,
        category: Optional[Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    @overload
    async def move(
        self,
        *,
        before: Snowflake,
        offset: int = ...,
        category: Optional[Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    @overload
    async def move(
        self,
        *,
        after: Snowflake,
        offset: int = ...,
        category: Optional[Snowflake] = ...,
        sync_permissions: bool = ...,
        reason: bool = ...,
    ) -> None: ...
    async def create_invite(
        self,
        *,
        reason: Optional[str] = ...,
        max_age: int = ...,
        max_uses: int = ...,
        temporary: bool = ...,
        unique: bool = ...,
    ) -> Invite: ...
    async def invites(self) -> List[Invite]: ...

class Messageable(metaclass=abc.ABCMeta):
    async def send(
        self,
        content: Optional[object] = ...,
        *,
        tts: bool = ...,
        embed: Optional[Embed] = ...,
        file: Optional[File] = ...,
        files: Optional[List[File]] = ...,
        delete_after: Optional[float] = ...,
        nonce: Optional[int] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
        reference: Optional[Union[Message, MessageReference]] = ...,
        mention_author: Optional[bool] = ...,
    ) -> Message: ...
    async def trigger_typing(self) -> None: ...
    def typing(self) -> Typing: ...
    async def fetch_message(self, id: int) -> Message: ...
    async def pins(self) -> List[Message]: ...
    def history(
        self,
        *,
        limit: Optional[int] = ...,
        before: Optional[Union[Snowflake, datetime.datetime]] = ...,
        after: Optional[Union[Snowflake, datetime.datetime]] = ...,
        around: Optional[Union[Snowflake, datetime.datetime]] = ...,
        oldest_first: Optional[bool] = ...,
    ) -> HistoryIterator: ...

class Connectable(metaclass=abc.ABCMeta):
    @overload
    async def connect(
        self, *, timeout: float = ..., reconnect: bool = ...
    ) -> VoiceClient: ...
    @overload
    async def connect(
        self, *, timeout: float = ..., reconnect: bool = ..., cls: Type[_VP] = ...
    ) -> _VP: ...
